program scene_generator;

uses graph, mygraph, svgabgi, stuff, crt, dos, myfile;

const
 aspecty=6;
 aspectx=5;
 COLORS=256;
 COLOR_RANGE=32;
 xmin=0;
 ymin=0;
 xmax=319;
 ymax=199;


var
 i,j,k,l:integer;

{/*****************************************************************************
* FUNC: void	shadedLine(left, right, y,
*									color1, color2, dither)
* 
* DESC:	Draw a horizontal line shaded from left to right color.
*****************************************************************************/}

procedure shadedLine(x1,x2,y,color1,color2,dither:integer);
var
 x, color:integer;
 rotColor, rotColorDelta,	rotColor1, rotColor2,
 rotDither, fractionBits:longint;
begin
	fractionBits:= 12;
	rotDither:= dither;rotDither:=rotDither shl fractionBits;
	rotColor1:= color1;rotColor1:=rotColor1 shl fractionBits;
	rotColor2:= color2;rotColor2:=rotColor2 shl fractionBits;
	rotColorDelta:= trunc((rotColor2 - rotColor1)	/ (x2 - x1));
  rotColor:= rotColor1;
  x:= x1;
 	while (x <= x2) do
   begin
		{* do horizontal line *}
		color:= (rotColor + (random(rotDither+1))) shr fractionBits;
		if (color > color2) then color:= color2;
    if (x>=0) and (x<=319) and (y>=0) and (y<=199) then
       mem[$A000:x+y*320]:=color;
    inc(x);
    rotColor:=rotColor+rotColorDelta;
	 end;
end;




{/*****************************************************************************
* FUNC: void	twoLines(x1Rel, x2Rel, yRel, x0, y0,
*							color1, color2, dither)
* 
* DESC:	Draw two shaded, horizontal lines above and below the center
*			(x0, y0) of an ellipse.
*****************************************************************************/}

procedure twoLines(x1Rel,x2Rel,yRel,x0,y0,color1,color2,dither:integer);
var
 y,x1,x2:integer;
begin
	x1:= x0 + x1Rel;	{* form the plotting coordinates from the *}
	x2:= x0 + x2Rel;	{* Relative values *}
	if (x2 >= Xmin) and (x1 <= Xmax) and (x1 <= x2) then
   begin
		{* first, the top line *}
		y:= y0 - yRel;
		shadedLine (x1, x2, y, color1, color2, dither);
		if (yRel<>0) then
     begin
			{* then the bottom line, if not redundant (yRel = 0) *}
			y:= y0 + yRel;
			shadedLine (x1, x2, y, color1, color2, dither);
     end;
   end;
end;



{/*****************************************************************************
* FUNC: void	ball(x0, y0, radius, color1, color2, dither)
* 
* DESC: Draw a shaded ball at x0,y0 with the given radius
*		and color range.  This is a Bresenham incremental ellipse algorithm.
*		Use the equation of an ellipse:
*
*					  	 2        2        2
*					(b x)  + (a y)	- (a b)  =  0
*
*		As you plot integer values of the function, this equation will not
*		be exactly zero.  This excess is the distance from the actual ellipse
*		to the integer approximation.  Whether this point is inside or outside
*		of the actual ellipse determines the position of the next point.
* 
*****************************************************************************/}

procedure ball(x0,y0,radius,color1,color2,dither:integer);
var
 c1,c2,colordelta,x,y:integer;
 a, aSquared, twoAsquared,
 b, bSquared, twoBsquared,
 d, dx, dy:longint;
begin
	x:= 0;
	y:= radius;
	a:= trunc(radius * AspectY / AspectX);
	aSquared:= a * a;
	twoAsquared:= 2 * aSquared;
	b:= radius;
	bSquared:= b * b;
	twoBsquared:= 2 * bSquared;

	{* this is the initial excess distance *}
	d:= trunc(bSquared - aSquared * b + aSquared / 4);

	dx:= 0;	{ these 2 guys are used to calculate d }
	dy:= twoAsquared * b;

	c1:= trunc((color2 + color1) / 2);	{* middle color *}
  c2:= c1;
	colorDelta:= trunc(COLOR_RANGE / 5);
	while (dx < dy) do         {* slow slope > -1 *}
   begin
		if (d > 0) then
     begin    {* midpoint is outside, go down one line *}
			dec(y);
			dy:=dy-twoAsquared;
			d :=d-dy;

			{* when you go down a line, fix the colors & draw the line *}
			if (c1 > color1) then c1 :=c1- colorDelta;	{* take the left and right colors *}
			if (c2 < color2) then c2 :=c2+ colorDelta;	{* to their respective limits *}
			if (colorDelta > 1) then colorDelta:=colorDelta shr 1; {* half the delta *}

		 	twoLines (-x, x, y, x0, y0, c1, c2, dither);

			inc(x);
			dx:=dx+ twoBsquared;
			d :=d+ bSquared + dx;
		 end else begin
			inc(x);	{* midpoint is inside *}
			dx:=dx+ twoBsquared;
			d :=d+ bSquared + dx;
     end;
	 end;
	{* reset d for the fast slope section *}
	d:=d+ trunc((3 * (aSquared - bSquared) / 2 - (dx + dy)) / 2);

	while (y > 0) do
   begin	{* fast slope < -1 *}
		if (d < 0) then
     begin		{* midpoint is outside *}
			inc(x);
			dx:=dx+ twoBsquared;
			d :=d+ dx;
		 end;
		dec(y);	{* midpoint is inside *}
		if (c1 > color1) then dec(c1);	{* continue to increase *}
		if (c2 < color2) then inc(c2);	{* the color range      *}

	 	twoLines (-x, x, y, x0, y0, c1, c2, dither);
		dy:=dy- twoAsquared;
		d:=d+ aSquared - dy;
	 end;
end;





begin
 graphitV;
 load_palette('default.pal');
 color_spread(224,255,0,10,0,63,40,15);
 setall(@color);
 {flushkey;
 repeat}
  ball(160,100,64,224,255,2);
 {until keypressed;}
 flushkey;
 readkey;
 closegraph;
end.

